<?php

namespace Tests\Feature;

use App\Models\Product;
use App\Models\Retailer;
use App\Models\User;
use Hash;
use Laravel\Sanctum\Sanctum;
use Tests\TestCase;

class ProductUpdateTest extends TestCase
{
    public function test_success_update_product_by_admin()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        $retailer = Retailer::factory()->create();
        $product = Product::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('/nova-api/products/' . $product->id . '?editing=true&editMode=update', [
            'name' => 'test_name',
            'retailer_id' => $retailer->id,
            'description' => 'defefef',
            'sku' => '000000',
            'price' => '0.00',
            'categories' => [1],
        ]);
        Product::where('retailer_id', $retailer->id)->delete();
        $retailer->delete();
        $product->delete();
        $user->delete();
        $response->assertStatus(200);
    }

    public function test_update_product_by_admin_with_the_same_sku_on_one_retailer()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        $retailer = Retailer::factory()->create();
        $product = Product::factory()->create([
            'retailer_id' => $retailer->id,
            'sku' => '000000',
        ]);
        $productForUpdate = Product::factory()->create([
            'retailer_id' => $retailer->id,
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('/nova-api/products/' . $productForUpdate->id . '?editing=true&editMode=update', [
            'name' => 'test_name',
            'retailer_id' => $product->retailer_id,
            'description' => 'defefef',
            'sku' => '000000',
            'price' => 0.00,
            'categories' => [1],
        ]);
        Product::where('retailer_id', $retailer->id)->delete();
        $retailer->delete();
        $product->delete();
        $productForUpdate->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['sku' => 'This sku is already in use.']);
    }

    public function test_update_product_by_admin_without_name()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        $product = Product::factory()->create();
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('/nova-api/products/' . $product->id . '?editing=true&editMode=update', [
            'retailer_id' => $retailer->id,
            'description' => 'defefef',
            'sku' => '000000',
            'price' => '0.00',
            'categories' => [1],
        ]);
        $product->delete();
        $retailer->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['name' => 'The Name field is required.']);
    }

    public function test_update_product_by_admin_without_retailer_id()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        $product = Product::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('/nova-api/products/' . $product->id . '?editing=true&editMode=update', [
            'name' => 'test_name',
            'description' => 'defefef',
            'sku' => '000000',
            'price' => '0.00',
            'categories' => [1],
        ]);
        $product->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['retailer_id' => 'The Retailer field is required.']);
    }

    public function test_update_product_by_admin_without_description()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        $product = Product::factory()->create();
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('/nova-api/products/' . $product->id . '?editing=true&editMode=update', [
            'name' => 'test_name',
            'retailer_id' => $retailer->id,
            'sku' => '000000',
            'price' => '0.00',
            'categories' => [1],
        ]);
        $product->delete();
        $retailer->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['description' => 'The Description field is required.']);
    }

    public function test_update_product_by_admin_without_sku()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        $product = Product::factory()->create();
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('/nova-api/products/' . $product->id . '?editing=true&editMode=update', [
            'name' => 'test_name',
            'retailer_id' => $retailer->id,
            'description' => 'defefef',
            'price' => '0.00',
            'categories' => [1],
        ]);
        $product->delete();
        $retailer->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['sku' => 'The SKU field is required.']);
    }

    public function test_update_product_by_admin_without_price()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        $product = Product::factory()->create();
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('/nova-api/products/' . $product->id . '?editing=true&editMode=update', [
            'name' => 'test_name',
            'retailer_id' => $retailer->id,
            'description' => 'defefef',
            'sku' => '000000',
            'categories' => [1],
        ]);
        $product->delete();
        $retailer->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['price' => 'The Price field is required.']);
    }

    public function test_success_update_product_by_analyst()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $product = Product::factory()->create();
        $response = $this->put('/nova-api/products/' . $product->id . '?editing=true&editMode=update', [
            'name' => 'test_name',
            'retailer_id' => $retailer->id,
            'description' => 'defefef',
            'sku' => '000000',
            'price' => '0.00',
            'categories' => [1],
        ]);
        Product::where('retailer_id', $retailer->id)->delete();
        $retailer->delete();
        $product->delete();
        $user->delete();
        $response->assertStatus(200);
    }

    public function test_update_product_by_analyst_with_the_same_sku_on_one_retailer()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        $retailer = Retailer::factory()->create();
        $product = Product::factory()->create([
            'retailer_id' => $retailer->id,
            'sku' => '000000',
        ]);
        $productForUpdate = Product::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('/nova-api/products/' . $productForUpdate->id . '?editing=true&editMode=update', [
            'name' => 'test_name',
            'retailer_id' => $retailer->id,
            'description' => 'defefef',
            'sku' => '000000',
            'price' => 0.00,
            'categories' => [1],
        ]);
        Product::where('retailer_id', $retailer->id)->delete();
        $retailer->delete();
        $product->delete();
        $productForUpdate->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['sku' => 'This sku is already in use.']);
    }

    public function test_update_product_by_analyst_without_name()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        $product = Product::factory()->create();
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('/nova-api/products/' . $product->id . '?editing=true&editMode=update', [
            'retailer_id' => $retailer->id,
            'description' => 'defefef',
            'sku' => '000000',
            'price' => '0.00',
            'categories' => [1],
        ]);
        $retailer->delete();
        $product->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['name' => 'The Name field is required.']);
    }

    public function test_update_product_by_analyst_without_retailer_id()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $product = Product::factory()->create();
        $response = $this->put('/nova-api/products/' . $product->id . '?editing=true&editMode=update', [
            'name' => 'test_name',
            'description' => 'defefef',
            'sku' => '000000',
            'price' => '0.00',
            'categories' => [1],
        ]);
        $product->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['retailer_id' => 'The Retailer field is required.']);
    }

    public function test_update_product_by_analyst_without_description()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        $product = Product::factory()->create();
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('/nova-api/products/' . $product->id . '?editing=true&editMode=update', [
            'name' => 'test_name',
            'retailer_id' => $retailer->id,
            'sku' => '000000',
            'price' => '0.00',
            'categories' => [1],
        ]);
        $product->delete();
        $retailer->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['description' => 'The Description field is required.']);
    }

    public function test_update_product_by_analyst_without_sku()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        $product = Product::factory()->create();
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('/nova-api/products/' . $product->id . '?editing=true&editMode=update', [
            'name' => 'test_name',
            'retailer_id' => $retailer->id,
            'description' => 'defefef',
            'price' => '0.00',
            'categories' => [1],
        ]);
        $product->delete();
        $retailer->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['sku' => 'The SKU field is required.']);
    }

    public function test_update_product_by_analyst_without_price()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        $product = Product::factory()->create();
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('/nova-api/products/' . $product->id . '?editing=true&editMode=update', [
            'name' => 'test_name',
            'retailer_id' => $retailer->id,
            'description' => 'defefef',
            'sku' => '000000',
            'categories' => [1],
        ]);
        $retailer->delete();
        $product->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['price' => 'The Price field is required.']);
    }

    public function test_update_product_by_common()
    {
        $user = User::factory()->create([
            'role' => 'common',
            'password' => Hash::make('123123123'),
        ]);
        $product = Product::factory()->create();
        $retailer = Retailer::factory()->create();
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->put('/nova-api/products/' . $product->id . '?editing=true&editMode=update', [
            'name' => 'test_name',
            'retailer_id' => $retailer->id,
            'description' => 'defefef',
            'sku' => '000000',
            'price' => '0.00',
            'categories' => [1],
        ]);
        $product->delete();
        $retailer->delete();
        $user->delete();
        $response->assertStatus(403);
    }

    public function test_update_product_by_anauthenticate()
    {
        $retailer = Retailer::factory()->create();
        $product = Product::factory()->create();
        $response = $this->put('/nova-api/products/' . $product->id . '?editing=true&editMode=update', [
            'name' => 'test_name',
            'retailer_id' => $retailer->id,
            'description' => 'defefef',
            'sku' => '000000',
            'price' => '0.00',
            'categories' => [1],
        ]);
        $product->delete();
        $retailer->delete();
        $response->assertStatus(401);
    }
}
