<?php

namespace Tests\Feature;

use App\Models\User;
use Hash;
use Laravel\Sanctum\Sanctum;
use Tests\TestCase;

class UserCreateTest extends TestCase
{
    public function test_success_create_retailer_by_admin()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->post('/nova-api/users?editing=true&editMode=create', [
            'name' => 'test_name',
            'email' => 'test_email@test.com',
            'role' => 'common',
            'password' => '123123123',
        ]);
        $user->delete();
        User::where('email', 'test_email@test.com')->delete();
        $response->assertStatus(201);
    }

    public function test_create_retailer_by_admin_without_name_of_user()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->post('/nova-api/users?editing=true&editMode=create', [
            'email' => 'test_email@test.com',
            'role' => 'common',
            'password' => '123123123',
        ]);
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['name' => 'The Name field is required.']);
    }

    public function test_create_retailer_by_admin_without_email()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->post('/nova-api/users?editing=true&editMode=create', [
            'name' => 'test_name',
            'role' => 'common',
            'password' => '123123123',
        ]);
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['email' => 'The Email field is required.']);
    }

    public function test_create_retailer_by_admin_without_role()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->post('/nova-api/users?editing=true&editMode=create', [
            'name' => 'test_name',
            'email' => 'test_email@test.com',
            'password' => '123123123',
        ]);
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['role' => 'The Role field is required.']);
    }

    public function test_create_retailer_by_admin_without_password()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->post('/nova-api/users?editing=true&editMode=create', [
            'name' => 'test_name',
            'email' => 'test_email@test.com',
            'role' => 'common',
        ]);
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors(['password' => 'The Password field is required.']);
    }

    public function test_create_retailer_by_admin_with_already_exist_email()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $testUser = User::factory()->create([
            'role' => 'common',
        ]);
        $response = $this->post('/nova-api/users?editing=true&editMode=create', [
            'name' => 'test_name',
            'email' => $user->email,
            'role' => 'common',
            'password' => '123123123',
        ]);
        $testUser->delete();
        $user->delete();
        $response->assertStatus(302)->assertSessionHasErrors([
            'email' => 'The Email has already been taken.',
        ]);
    }

    public function test_create_retailer_by_analyst()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->post('/nova-api/users?editing=true&editMode=create', [
            'name' => 'test_name',
            'email' => 'test_email@test.com',
            'role' => 'common',
            'password' => '123123123',
        ]);
        $user->delete();
        $response->assertStatus(403);
    }

    public function test_create_retailer_by_common()
    {
        $user = User::factory()->create([
            'role' => 'common',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $response = $this->post('/nova-api/users?editing=true&editMode=create', [
            'name' => 'test_name',
            'email' => 'test_email@test.com',
            'role' => 'common',
            'password' => '123123123',
        ]);
        $user->delete();
        $response->assertStatus(403);
    }

    public function test_create_retailer_by_unauthenticate_user()
    {
        $response = $this->post('/nova-api/users?editing=true&editMode=create', [
            'name' => 'test_name',
            'email' => 'test_email@test.com',
            'role' => 'common',
            'password' => '123123123',
        ]);
        $response->assertStatus(401);
    }
}
